@@ -1,12 +1,12 @@
 use tree_sitter::{Node, Parser, Point, Tree};
-use tree_sitter_generate::{generate_parser_for_grammar, load_grammar_file};
+use tree_sitter_generate::load_grammar_file;
 
 use super::{
     get_random_edit,
     helpers::fixtures::{fixtures_dir, get_language, get_test_language},
     Rand,
 };
-use crate::parse::perform_edit;
+use crate::{parse::perform_edit, tests::generate_parser};
 
 const JSON_EXAMPLE: &str = r#"
 
@@ -306,6 +306,76 @@ fn test_parent_of_zero_width_node() {
     assert_eq!(parent, script_element);
 }
 
+#[test]
+fn test_next_sibling_of_zero_width_node() {
+    let grammar_json = load_grammar_file(
+        &fixtures_dir()
+            .join("test_grammars")
+            .join("next_sibling_from_zwt")
+            .join("grammar.js"),
+        None,
+    )
+    .unwrap();
+
+    let (parser_name, parser_code) = generate_parser(&grammar_json).unwrap();
+
+    let mut parser = Parser::new();
+    let language = get_test_language(&parser_name, &parser_code, None);
+    parser.set_language(&language).unwrap();
+
+    let tree = parser.parse("abdef", None).unwrap();
+
+    let root_node = tree.root_node();
+    let missing_c = root_node.child(2).unwrap();
+    assert!(missing_c.is_missing());
+    assert_eq!(missing_c.kind(), "c");
+    let node_d = root_node.child(3).unwrap();
+    assert_eq!(missing_c.next_sibling().unwrap(), node_d);
+
+    let prev_sibling = node_d.prev_sibling().unwrap();
+    assert_eq!(prev_sibling, missing_c);
+}
+
+#[test]
+fn test_first_child_for_offset() {
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("javascript")).unwrap();
+    let tree = parser.parse("x10 + 100", None).unwrap();
+    let sum_node = tree.root_node().child(0).unwrap().child(0).unwrap();
+
+    assert_eq!(
+        sum_node.first_child_for_byte(0).unwrap().kind(),
+        "identifier"
+    );
+    assert_eq!(
+        sum_node.first_child_for_byte(1).unwrap().kind(),
+        "identifier"
+    );
+    assert_eq!(sum_node.first_child_for_byte(3).unwrap().kind(), "+");
+    assert_eq!(sum_node.first_child_for_byte(5).unwrap().kind(), "number");
+}
+
+#[test]
+fn test_first_named_child_for_offset() {
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("javascript")).unwrap();
+    let tree = parser.parse("x10 + 100", None).unwrap();
+    let sum_node = tree.root_node().child(0).unwrap().child(0).unwrap();
+
+    assert_eq!(
+        sum_node.first_named_child_for_byte(0).unwrap().kind(),
+        "identifier"
+    );
+    assert_eq!(
+        sum_node.first_named_child_for_byte(1).unwrap().kind(),
+        "identifier"
+    );
+    assert_eq!(
+        sum_node.first_named_child_for_byte(3).unwrap().kind(),
+        "number"
+    );
+}
+
 #[test]
 fn test_node_field_name_for_child() {
     let mut parser = Parser::new();
@@ -493,8 +563,7 @@ fn test_node_named_child() {
 
 #[test]
 fn test_node_named_child_with_aliases_and_extras() {
-    let (parser_name, parser_code) =
-        generate_parser_for_grammar(GRAMMAR_WITH_ALIASES_AND_EXTRAS).unwrap();
+    let (parser_name, parser_code) = generate_parser(GRAMMAR_WITH_ALIASES_AND_EXTRAS).unwrap();
 
     let mut parser = Parser::new();
     parser
@@ -684,6 +753,13 @@ fn test_node_descendant_for_range() {
 
         assert_eq!(child, child2);
     }
+
+    // Negative test, start > end
+    assert_eq!(array_node.descendant_for_byte_range(1, 0), None);
+    assert_eq!(
+        array_node.descendant_for_point_range(Point::new(6, 8), Point::new(6, 7)),
+        None
+    );
 }
 
 #[test]
@@ -761,6 +837,20 @@ fn test_node_is_extra() {
     assert!(comment_node.is_extra());
 }
 
+#[test]
+fn test_node_is_error() {
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("javascript")).unwrap();
+    let tree = parser.parse("foo(", None).unwrap();
+    let root_node = tree.root_node();
+    assert_eq!(root_node.kind(), "program");
+    assert!(root_node.has_error());
+
+    let child = root_node.child(0).unwrap();
+    assert_eq!(child.kind(), "ERROR");
+    assert!(child.is_error());
+}
+
 #[test]
 fn test_node_sexp() {
     let mut parser = Parser::new();
@@ -780,7 +870,7 @@ fn test_node_sexp() {
 
 #[test]
 fn test_node_field_names() {
-    let (parser_name, parser_code) = generate_parser_for_grammar(
+    let (parser_name, parser_code) = generate_parser(
         r#"
         {
             "name": "test_grammar_with_fields",
@@ -890,7 +980,7 @@ fn test_node_field_names() {
 
 #[test]
 fn test_node_field_calls_in_language_without_fields() {
-    let (parser_name, parser_code) = generate_parser_for_grammar(
+    let (parser_name, parser_code) = generate_parser(
         r#"
         {
             "name": "test_grammar_with_no_fields",
@@ -948,7 +1038,7 @@ fn test_node_is_named_but_aliased_as_anonymous() {
     )
     .unwrap();
 
-    let (parser_name, parser_code) = generate_parser_for_grammar(&grammar_json).unwrap();
+    let (parser_name, parser_code) = generate_parser(&grammar_json).unwrap();
 
     let mut parser = Parser::new();
     let language = get_test_language(&parser_name, &parser_code, None);
@@ -1026,6 +1116,31 @@ fn test_node_numeric_symbols_respect_simple_aliases() {
     assert_eq!(unary_minus_node.kind_id(), binary_minus_node.kind_id());
 }
 
+#[test]
+fn test_hidden_zero_width_node_with_visible_child() {
+    let code = r"
+class Foo {
+  std::
+private:
+  std::string s;
+};
+";
+
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("cpp")).unwrap();
+    let tree = parser.parse(code, None).unwrap();
+    let root = tree.root_node();
+
+    let class_specifier = root.child(0).unwrap();
+    let field_decl_list = class_specifier.child_by_field_name("body").unwrap();
+    let field_decl = field_decl_list.named_child(0).unwrap();
+    let field_ident = field_decl.child_by_field_name("declarator").unwrap();
+    assert_eq!(
+        field_decl.child_with_descendant(field_ident).unwrap(),
+        field_ident
+    );
+}
+
 fn get_all_nodes(tree: &Tree) -> Vec<Node> {
     let mut result = Vec::new();
     let mut visited_children = false;
